/*
    This program tries to animate a cellular automata. For each
    "time step" the array of states is shiffted up, so the oldest
    state is lost off the top of the window, and a new state is
    entered at the bottom row of the window.
*/
import java.applet.Applet;
import java.awt.*;
import java.awt.image.*;

public class CellularAutomataAnimation extends Applet implements Runnable
{
   int SIZE = 512;
   Image memImage;
   Thread animate;
   int[] pixels;
   MemoryImageSource mis;
   int[] oldState;
   int[] newState;
   int[] tempState;

   public void init()
   {
      // create a random transition rule
      //transitions = new int[32];
      //for (int i = 0; i < 32; i++){transitions[i] = (int)(2*Math.random());}

      pixels = new int[SIZE*SIZE];  // the image array
      ca( pixels );  //calculate the initial cellular automata
      mis = new MemoryImageSource(SIZE, SIZE, pixels, 0, SIZE);
      mis.setAnimated(true);
      memImage = createImage( mis );
      animate = new Thread(this);
      animate.start();
   }//init

   public void update(Graphics g){ paint(g); }

   public void paint(Graphics g)
   {
      g.drawImage(memImage, 0, 0, null);
   }//paint

   // do the animation
   public void run()
   {
      while(true)
      {
         // move all the rows up in the pixel array
         for(int i = 0; i < SIZE*(SIZE - 1); i++)
            pixels[i] = pixels[i+SIZE];
         // now generate a new bottom row for the pixel array
         for (int col = 0; col < oldState.length; col++)
         {
            // compute one cell in the new state
            int index = 16*oldState[(col-2+oldState.length)%oldState.length] +
                         8*oldState[(col-1+oldState.length)%oldState.length] +
                         4*oldState[(col+0+oldState.length)%oldState.length] +
                         2*oldState[(col+1+oldState.length)%oldState.length] +
                         1*oldState[(col+2+oldState.length)%oldState.length];
            newState[col] = transitions[ index ];
            // plot a point when the state is 1
            if (newState[col] == 0)
               pixels[SIZE*(SIZE - 1)+col] = 0xFF000000;
            else
               pixels[SIZE*(SIZE - 1)+col] = 0xFFFFFFFF;
         }
         tempState = oldState;  // swap the oldState and the newState
         oldState = newState;
         newState = tempState;
         mis.newPixels();
         repaint();
         try{animate.sleep(5);}catch(InterruptedException e){}
      }
   }//run

   // compute the initial cellular automata
   public void ca(int[] pixels)
   {
      oldState = new int[SIZE];
      newState = new int[SIZE];

      // create a random initial state
      for (int i = 0; i < oldState.length; i++)
         oldState[i] = (int)( 2*Math.random() );

      // run the cellular automata SIZE times
      for (int row = 0; row < SIZE; row++)
      {
         // compute the new state from the previous state
         for (int col = 0; col < oldState.length; col++)
         {
            // compute one cell in the new state
            int index = 16*oldState[(col-2+oldState.length)%oldState.length] +
                         8*oldState[(col-1+oldState.length)%oldState.length] +
                         4*oldState[(col+0+oldState.length)%oldState.length] +
                         2*oldState[(col+1+oldState.length)%oldState.length] +
                         1*oldState[(col+2+oldState.length)%oldState.length];
            newState[col] = transitions[ index ];
            // plot a point when the state is 1
            if (newState[col] == 0)
               pixels[row*SIZE+col] = 0xFF000000;
            else
               pixels[row*SIZE+col] = 0xFFFFFFFF;
         }
         tempState = oldState;
         oldState = newState;
         newState = tempState;
      }
   }//ca

   // Here is the array that contains the transition rules
   // for the cellular automata. This particular rule comes
   // from "The Pattern Book" by Clifford Pickover, pages 283-284.
   int[] transitions = {0,0,0,0,1,0,0,1,1,1,0,0,0,1,1,1,0,0,1,1,1,1,0,1,1,1,0,0,0,0,1,1};
   //int[] transitions;  // used for the randomly generated transition rules

}//CellularAutomataAnimation